@@ -31,32 +31,41 @@ class BaseClass(estruttura.UserStructure):
         return self
 
 
-class ImmutableClass(BaseClass, estruttura.UserImmutableStructure):
+class ImmutableClass(estruttura.UserImmutableStructure, BaseClass):
     """Immutable attribute class."""
 
-    def _hash(self):
-        return hash(tuple(sorted(self._internal.items(), key=lambda i: id(i[0]))))
+    __slots__ = ("__hash",)
+
+    def __cache_hash__(self, hash_):
+        self.__hash = hash_
+
+    def __retrieve_hash__(self):
+        try:
+            return self.__hash
+        except AttributeError:
+            return None
+
+    def _do_copy(self):
+        return copy.copy(self)
 
     def _do_update(self, inserts, deletes, updates_old, updates_new, updates_and_inserts, all_updates):
-        new_internal = dict((k, v) for k, v in self._internal.items() if k not in deletes)
+        new_internal = dict(
+            (k, v) for k, v in self._internal.items() if k not in deletes and k not in updates_and_inserts
+        )
         new_internal.update(updates_and_inserts)
-        new_self = copy.copy(self)
-        new_self._internal = new_internal
-        return new_self
+        self._internal = new_internal
 
     def _do_clear(self):
-        return type(self)()
+        self._internal = {}
 
 
-class MutableClass(BaseClass, estruttura.UserMutableStructure):
+class MutableClass(estruttura.UserMutableStructure, BaseClass):
     """Mutable attribute class."""
 
     def _do_update(self, inserts, deletes, updates_old, updates_new, updates_and_inserts, all_updates):
         for key in deletes:
             del self._internal[key]
         self._internal.update(updates_and_inserts)
-        return self
 
     def _do_clear(self):
         self._internal.clear()
-        return self
